Enum virtio_vsock::packet::Error
source · pub enum Error {
DescriptorChainTooShort,
DescriptorLengthTooSmall,
DescriptorLengthTooLong,
InvalidHeaderInputSize(usize),
InvalidHeaderLen(u32),
InvalidMemoryAccess(GuestMemoryError),
InvalidVolatileAccess(VolatileMemoryError),
UnexpectedReadOnlyDescriptor,
UnexpectedWriteOnlyDescriptor,
}Expand description
Vsock packet parsing errors.
Variants
DescriptorChainTooShort
Too few descriptors in a descriptor chain.
DescriptorLengthTooSmall
Descriptor that was too short to use.
DescriptorLengthTooLong
Descriptor that was too long to use.
InvalidHeaderInputSize(usize)
The slice for creating a header has an invalid length.
InvalidHeaderLen(u32)
The len header field value exceeds the maximum allowed data size.
InvalidMemoryAccess(GuestMemoryError)
Invalid guest memory access.
InvalidVolatileAccess(VolatileMemoryError)
Invalid volatile memory access.
UnexpectedReadOnlyDescriptor
Read only descriptor that protocol says to write to.
UnexpectedWriteOnlyDescriptor
Write only descriptor that protocol says to read from.